#include "line.h"

#include <zephyr/sys/printk.h>

#include "timing.h"

namespace ch81 {

constexpr uint8_t kHalt = 118;
constexpr uint16_t kWhite = 0xFFFFU;
constexpr uint16_t kBlack = 0x0;

static const uint8_t kFont[] = {
// echo $(od -w10000 -A none -t u1 ../../zephyr/blobs/zx81font.rom) | tr ' ' ','
#include "zx81font.h"
};

__attribute__((optimize(2))) const uint8_t* Render(const uint8_t* dfile, int y,
                                                   uint16_t* into, int* wrote) {
  const uint8_t* font = &kFont[y];
  uint16_t* base = into;

  // Assuming the transfer is triggered at the end of the back porch.

  constexpr int display_nanos = kLineNanos
                                // Front porch minimum
                                - kFrontPorchNanos - kBackPorchNanos -
                                kHsyncNanos;
  constexpr int text_nanos = kTextWidth * kNanosPerByte;
  constexpr int border_nanos = display_nanos - text_nanos;

  constexpr int border = border_nanos / (kNanosPerByte * 2);
  static_assert(border >= 2);
  constexpr int right_border = border / 2;
  constexpr int left_border = border - right_border;
  static_assert(left_border >= 1 && right_border >= 1);
  // static_assert(right_border == 3);
  for (int i = 0; i < left_border; ++i) {
    *into++ = kWhite;
  }

  static_assert(kTextWidth % 2 == 0);
  for (;;) {
    if (uint8_t ch0 = dfile[0]; ch0 != kHalt) {
      if (uint8_t ch1 = dfile[1]; ch1 != kHalt) {
        *into++ = ((font[ch0 * 8] << 8) | font[ch1 * 8]) ^ kWhite;
        dfile += 2;
      } else {
        *into++ = (font[ch0 * 8] << 8) ^ kWhite;
        dfile += 2;
        break;
      }
    } else {
      ++dfile;
      break;
    }
  }

  // Pad out the text and add the right border.
  for (int i = into - base - left_border; i < kTextWidth / 2; ++i) {
    *into++ = kWhite;
  }
  for (int i = 0; i < right_border - 1; ++i) {
    *into++ = kWhite;
  }
  // Front porch.
  *into++ = kWhite ^ 1;
  *wrote = into - base;

  return dfile;
}

}  // namespace ch81
