#pragma once

#include <hal_ch32fun.h>
#include <zephyr/drivers/pwm.h>

class PwmChannel {
 public:
  virtual ~PwmChannel() = default;

  virtual void SetPeriod(uint16_t period) = 0;
  virtual void SetPulse(uint16_t pulse) = 0;
  virtual void AwaitTop() = 0;
};

template <pwm_dt_spec Spec, uintptr_t Regs>
class DevicetreePwmChannel final : public PwmChannel {
 public:
  void SetPeriod(uint16_t reload) override { regs()->ATRLR = reload; };

  void SetPulse(uint16_t pulse) override {
    if constexpr (Spec.channel == 0) {
      regs()->CH1CVR = pulse;
    } else if constexpr (Spec.channel == 1) {
      regs()->CH2CVR = pulse;
    } else if constexpr (Spec.channel == 2) {
      regs()->CH3CVR = pulse;
    } else if constexpr (Spec.channel == 3) {
      regs()->CH4CVR = pulse;
    }
  }

  void AwaitTop() override {
    while ((regs()->INTFR & TIM2_INTFR_UIF) == 0) {
    }
    regs()->INTFR = 0;
  }

  void AwaitCompare() {
    constexpr uint16_t mask = TIM1_INTFR_CC1IF << Spec.channel;
    regs()->INTFR &= ~mask;
    while ((regs()->INTFR & mask) == 0) {
    }
  }

 private:
  static_assert(Spec.channel >= 0 && Spec.channel < 4);

  GPTM_TypeDef* regs() const { return reinterpret_cast<GPTM_TypeDef*>(Regs); }
};
