#pragma once

#include <zephyr/kernel.h>

namespace ch81 {

constexpr int kTextWidth = 32;
constexpr int kTextHeight = 24;

// Lowest prescaler that gives no rounding error on kNanosPerPulse.
constexpr int kPwmPrescaler = 6;
constexpr int kNanosPerPulse = kPwmPrescaler * 1000 / 48;
static_assert(kNanosPerPulse == 125);

// NTSC timings based on "Programming 32-bit Microcontrollers in C" by Di Jasio.
constexpr int kLineNanos = 63'500;
constexpr int kHsyncNanos = 5'040;
constexpr int kFrontPorchNanos = 1'650;
constexpr int kBackPorchNanos = 4'480;
static_assert(kLineNanos / kNanosPerPulse < 0xFFFF);
constexpr int kDisplayNanos =
    kLineNanos - kBackPorchNanos - kHsyncNanos - kFrontPorchNanos;

constexpr int kSpiClockHz = DT_PROP(DT_NODELABEL(cpu0), clock_frequency);

// Pick the dot clock based on using ~85 % of the display area.
constexpr int kDotClockKHz =
    kTextWidth * 8 * 1'000'000 / kDisplayNanos * 100 / 85;
constexpr int kSpiPrescaler = kSpiClockHz / 1000 / kDotClockKHz;
static_assert(kSpiPrescaler >= 5);
constexpr int kNanosPerByte =
    kSpiPrescaler * 8 * 1000 / (kSpiClockHz / 1'000'000);

constexpr int kLines = 525 / 2;
constexpr int kPreEqualizingLines = 3;
constexpr int kVsyncLines = 3;
constexpr int kPostEqualizingLines = 3;
constexpr int kPreBorderLines = 11;
constexpr int kTextLines = kTextHeight * 8;
constexpr int kBorderLines = kLines - kPreEqualizingLines - kVsyncLines -
                             kPostEqualizingLines - kPreBorderLines -
                             kTextLines;
constexpr int kTopBorderLines = kBorderLines * 2 / 3;
constexpr int kBottomBorderLines = kBorderLines - kTopBorderLines;
static_assert(kPreEqualizingLines + kVsyncLines + kPostEqualizingLines +
                  kPreBorderLines + kTopBorderLines + kTextLines +
                  kBottomBorderLines ==
              kLines);

}  // namespace ch81
